from aioredis.commands import Redis, Pipeline
from aioredis.connection import RedisConnection
from aioredis.pool import ConnectionsPool

from typing import Union


class Script(object):
    """An executable Lua script object returned by ``register_script``"""

    def __init__(self, registered_client: Union[ConnectionsPool, RedisConnection, Redis], script: str):
        self.registered_client = registered_client
        self.script = script

    async def __call__(self, keys: list=list(), args: list=list(),
                       client: Union[Pipeline, ConnectionsPool, RedisConnection, Redis]=None):
        """Execute the script, passing any required ``args``"""
        if client is None:
            client = self.registered_client
        if isinstance(client, (RedisConnection, ConnectionsPool)):
            args = tuple(keys) + tuple(args)
            return await client.execute('eval', self.script, len(keys), *args)
        if isinstance(client, Pipeline):
            return client.eval(self.script, keys=keys, args=args)
        else:
            return await client.eval(self.script, keys=keys, args=args)



async def wait_for_synced_slaves(redis, count: int, timeout: int):
    synced = await redis.execute('WAIT', count, timeout)
    if synced < count:
        raise NotEnoughSyncedSlavesError('There are only {} synced slaves. Required {}'.format(synced, count))


def create_chunks(items, chunk_size):
    for i in range(0, len(items), chunk_size):
        yield items[i:i + chunk_size]


async def async_iterate_to_list(cmd) -> list:
    lst = []
    async for i in cmd:
        lst.append(i)
    return lst


class NotEnoughSyncedSlavesError(Exception):
    pass




from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='aio-py-rq',
    version='1.0.2',
    packages=['aiopyrq'],
    url='https://github.com/heureka/aio-py-rq',
    license='MIT',
    author='Heureka.cz',
    author_email='podpora@heureka.cz',
    description='Redis queue for Asynchronous Python',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        "aioredis>1.3.1"
    ]
)